@@ -9,6 +9,7 @@ from ecowitt2mqtt.mqtt import async_publish_payload
 
 DEFAULT_AIOHTTP_ENDPOINT = "/data/report"
 DEFAULT_AIOHTTP_PORT = 8080
+DEFAULT_HASS_DISCOVERY_PREFIX = "homeassistant"
 DEFAULT_LOG_LEVEL_STRING = "INFO"
 DEFAULT_MQTT_PORT = 1883
 
@@ -72,7 +73,11 @@ def get_arguments() -> argparse.Namespace:
     )
     parser.add_argument(
         "--hass-discovery-prefix",
-        help="The Home Assistant discovery prefix to use (default: homeassistant)",
+        default=DEFAULT_HASS_DISCOVERY_PREFIX,
+        help=(
+            "The Home Assistant discovery prefix to use "
+            f"(default: {DEFAULT_HASS_DISCOVERY_PREFIX})"
+        ),
     )
 
     # Web Server
@@ -96,17 +101,30 @@ def get_arguments() -> argparse.Namespace:
 
     # Misc.
     parser.add_argument(
-        "--unit-system",
+        "--raw-data",
+        action="store_const",
+        const=True,
+        help="Return raw data (don't attempt to translate any values)",
+    )
+    parser.add_argument(
+        "--input-unit-system",
         action="store",
         default=UNIT_SYSTEM_IMPERIAL,
         type=str,
-        help=f"The unit system to use (default: {UNIT_SYSTEM_IMPERIAL})",
+        help=f"The input unit system used by the device (default: {UNIT_SYSTEM_IMPERIAL})",
+    )
+    parser.add_argument(
+        "--output-unit-system",
+        action="store",
+        default=UNIT_SYSTEM_IMPERIAL,
+        type=str,
+        help=f"The unit system to use in output (default: {UNIT_SYSTEM_IMPERIAL})",
     )
 
     return parser.parse_args()
 
 
-def main():
+def main() -> None:
     """Run."""
     args = get_arguments()
     logging.basicConfig(level=getattr(logging, args.log_level))
@@ -114,7 +132,7 @@ def main():
     LOGGER.debug("Using arguments: %s", args)
 
     app = web.Application()
-    app.add_routes([web.post(args.endpoint, async_publish_payload)])
+    app.add_routes([web.post(args.endpoint, async_publish_payload)])  # type: ignore
 
     app["args"] = args
     app["hass_discovery_managers"] = {}
