@@ -1,7 +1,7 @@
 """Define aiohttp routes."""
 import asyncio
 import json
-from typing import Optional, Union
+from typing import Any, Dict, Union
 
 from aiohttp import web
 from asyncio_mqtt import Client, MqttError
@@ -10,8 +10,10 @@ from ecowitt2mqtt.const import LOGGER
 from ecowitt2mqtt.data import DataProcessor
 from ecowitt2mqtt.hass import HassDiscovery
 
+DEFAULT_MAX_MQTT_CALLS = 10
 
-def _generate_payload(data: Union[dict, float, str]) -> bytes:
+
+def _generate_payload(data: Union[Dict[str, Any], float, str]) -> bytes:
     """Generate a binary MQTT payload from input data."""
     if isinstance(data, dict):
         return json.dumps(data).encode("utf-8")
@@ -23,7 +25,7 @@ def _generate_payload(data: Union[dict, float, str]) -> bytes:
 
 
 async def _async_publish_to_hass_discovery(
-    client: Client, data: dict, discovery_manager: HassDiscovery
+    client: Client, data: Dict[str, Any], discovery_manager: HassDiscovery
 ) -> None:
     """Publish data to appropriate topics for Home Assistant Discovery."""
     LOGGER.debug("Publishing according to Home Assistant MQTT Discovery standard")
@@ -32,8 +34,8 @@ async def _async_publish_to_hass_discovery(
         async with client:
             tasks = []
             for key, value in data.items():
-                config_payload = discovery_manager.get_config_payload(key)
-                config_topic = discovery_manager.get_config_topic(key)
+                config_payload = discovery_manager.get_config_payload(key, value)
+                config_topic = discovery_manager.get_config_topic(key, value)
 
                 tasks.append(
                     client.publish(config_topic, _generate_payload(config_payload))
@@ -46,7 +48,9 @@ async def _async_publish_to_hass_discovery(
                 )
                 tasks.append(
                     client.publish(
-                        config_payload["state_topic"], _generate_payload(value)
+                        config_payload["state_topic"],
+                        _generate_payload(value),
+                        retain=True,
                     )
                 )
 
@@ -59,7 +63,7 @@ async def _async_publish_to_hass_discovery(
 
 
 async def _async_publish_to_topic(
-    client: Client, data: dict, *, topic: Optional[str] = None
+    client: Client, data: Dict[str, Any], topic: str
 ) -> None:
     """Publish data to a single MQTT topic."""
     LOGGER.debug("Publishing entire device payload to single topic: %s", topic)
@@ -74,14 +78,14 @@ async def _async_publish_to_topic(
     LOGGER.info("Published to %s: %s", topic, data)
 
 
-async def async_publish_payload(request: web.Request):
+async def async_publish_payload(request: web.Request) -> None:
     """Define the endpoint for the Ecowitt device to post data to."""
     args = request.app["args"]
 
     payload = dict(await request.post())
     LOGGER.debug("Received data from Ecowitt device: %s", payload)
 
-    data_processor = DataProcessor(payload, args.unit_system)
+    data_processor = DataProcessor(payload, args)
     data = data_processor.generate_data()
 
     client = Client(
@@ -90,32 +94,23 @@ async def async_publish_payload(request: web.Request):
         username=args.mqtt_username,
         password=args.mqtt_password,
         logger=LOGGER,
+        max_concurrent_outgoing_calls=DEFAULT_MAX_MQTT_CALLS,
     )
 
     if args.hass_discovery:
         discovery_managers = request.app["hass_discovery_managers"]
 
-        try:
-            discovery_manager = discovery_managers[data_processor.unique_id]
-        except KeyError:
-            if args.hass_discovery_prefix:
-                discovery_manager = discovery_managers[
-                    data_processor.unique_id
-                ] = HassDiscovery(
-                    data_processor.unique_id,
-                    args.unit_system,
-                    discovery_prefix=args.hass_discovery_prefix,
-                )
-            else:
-                discovery_manager = discovery_managers[
-                    data_processor.unique_id
-                ] = HassDiscovery(data_processor.unique_id, args.unit_system)
+        if data_processor.device.unique_id in discovery_managers:
+            discovery_manager = discovery_managers[data_processor.device.unique_id]
+        else:
+            discovery_manager = HassDiscovery(data_processor.device, args)
+            discovery_managers[data_processor.device.unique_id] = discovery_manager
 
         await _async_publish_to_hass_discovery(client, data, discovery_manager)
     else:
         if args.mqtt_topic:
             topic = args.mqtt_topic
         else:
-            topic = f"ecowitt2mqtt/{data_processor.unique_id}"
+            topic = f"ecowitt2mqtt/{data_processor.device.unique_id}"
 
-        await _async_publish_to_topic(client, data, topic=topic)
+        await _async_publish_to_topic(client, data, topic)
